@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
-
-# Authors: Alejandro J. Cura <alecu@canonical.com>
-# Authors: Natalia B. Bidart <nataliabidart@canonical.com>
 #
-# Copyright 2010 Canonical Ltd.
+# Copyright 2010-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -26,49 +23,51 @@ from collections import defaultdict
 from functools import wraps
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-# No name 'is_link' in module 'ubuntuone.platform'
-# pylint: disable=E0611, F0401
-from ubuntuone.platform import is_link
 from ubuntuone.platform.credentials import CredentialsManagementTool
-# pylint: enable=E0611, F0401
 
 from ubuntuone.controlpanel import sd_client, replication_client
 from ubuntuone.controlpanel.logger import setup_logging, log_call
-# pylint: disable=W0611
-from ubuntuone.controlpanel.web_client import (UnauthorizedError,
-    web_client_factory, WebClientError)
-# pylint: enable=W0611
+from ubuntuone.controlpanel.web_client import (
+    UnauthorizedError,
+    WebClient,
+    WebClientError,
+)
 
 logger = setup_logging('backend')
 
-ACCOUNT_API = "account/"
-QUOTA_API = "quota/"
-DEVICES_API = "1.0/devices/"
-DEVICE_REMOVE_API = "1.0/devices/remove/%s/%s"
-DEVICE_TYPE_PHONE = "Phone"
-DEVICE_TYPE_COMPUTER = "Computer"
-AUTOCONNECT_KEY = 'autoconnect'
-SHOW_ALL_NOTIFICATIONS_KEY = 'show_all_notifications'
-SHARE_AUTOSUBSCRIBE_KEY = 'share_autosubscribe'
-UDF_AUTOSUBSCRIBE_KEY = 'udf_autosubscribe'
-LIMIT_BW_KEY = 'limit_bandwidth'
-UPLOAD_KEY = "max_upload_speed"
-DOWNLOAD_KEY = "max_download_speed"
+ACCOUNT_API = u"account/"
+DEVICES_API = u"1.0/devices/"
+DEVICE_REMOVE_API = u"1.0/devices/remove/%s/%s"
+QUOTA_API = u"quota/"
 
-FILE_SYNC_DISABLED = 'file-sync-disabled'
-FILE_SYNC_DISCONNECTED = 'file-sync-disconnected'
-FILE_SYNC_ERROR = 'file-sync-error'
-FILE_SYNC_IDLE = 'file-sync-idle'
-FILE_SYNC_STARTING = 'file-sync-starting'
-FILE_SYNC_STOPPED = 'file-sync-stopped'
-FILE_SYNC_SYNCING = 'file-sync-syncing'
-FILE_SYNC_UNKNOWN = 'file-sync-unknown'
+DEVICE_TYPE_PHONE = u"Phone"
+DEVICE_TYPE_COMPUTER = u"Computer"
+
+AUTOCONNECT_KEY = u'autoconnect'
+SHOW_ALL_NOTIFICATIONS_KEY = u'show_all_notifications'
+SHARE_AUTOSUBSCRIBE_KEY = u'share_autosubscribe'
+UDF_AUTOSUBSCRIBE_KEY = u'udf_autosubscribe'
+LIMIT_BW_KEY = u'limit_bandwidth'
+UPLOAD_KEY = u'max_upload_speed'
+DOWNLOAD_KEY = u'max_download_speed'
+
+FILE_SYNC_DISABLED = u'file-sync-disabled'
+FILE_SYNC_DISCONNECTED = u'file-sync-disconnected'
+FILE_SYNC_ERROR = u'file-sync-error'
+FILE_SYNC_IDLE = u'file-sync-idle'
+FILE_SYNC_STARTING = u'file-sync-starting'
+FILE_SYNC_STOPPED = u'file-sync-stopped'
+FILE_SYNC_SYNCING = u'file-sync-syncing'
+FILE_SYNC_UNKNOWN = u'file-sync-unknown'
 
 MSG_KEY = 'message'
 STATUS_KEY = 'status'
 
 CONTACTS_PKG = 'thunderbird-couchdb'
 
+UBUNTUONE_FROM_OAUTH = u'https://one.ubuntu.com/api/1.0/from_oauth/'
+UBUNTUONE_LINK = u'https://one.ubuntu.com/'
+
 
 def append_path_sep(path):
     """If 'path' does not end with the path separator, append it."""
@@ -135,10 +134,14 @@ class ControlBackend(object):
 
         self.login_client = CredentialsManagementTool()
         self.sd_client = sd_client.SyncDaemonClient()
-        self.wc = web_client_factory(self.get_credentials)
+        self.wc = WebClient(self.get_credentials)
 
         logger.info('ControlBackend: instance started.')
 
+    def get_home_dir(self):
+        """Return a defer with the home_dir from ubuntu one client."""
+        return self.sd_client.get_home_dir()
+
     def _process_file_sync_status(self, status):
         """Process raw file sync status into custom format.
 
@@ -289,11 +292,28 @@ class ControlBackend(object):
 
         returnValue(local_device)
 
+    @inlineCallbacks
     def _process_path(self, path):
         """Trim 'path' so the '~' is removed."""
-        home = os.path.expanduser('~')
-        result = path.replace(os.path.join(home, ''), '')
-        return result
+        home_dir = yield self.get_home_dir()
+        if not home_dir.endswith(os.path.sep):
+            home_dir += os.path.sep
+        if path.startswith(home_dir):
+            path = path[len(home_dir):]
+        returnValue(path)
+
+    @inlineCallbacks
+    def build_signed_iri(self, iri):
+        """Returned an OAuth signed iri."""
+        credentials = None
+        if iri.startswith(UBUNTUONE_LINK):
+            credentials = yield self.get_credentials()
+
+        if credentials:
+            parameters = {'next': iri}
+            iri = yield self.wc.build_signed_iri(UBUNTUONE_FROM_OAUTH,
+                                                 parameters)
+        returnValue(iri)
 
     @inlineCallbacks
     def get_credentials(self):
@@ -316,10 +336,23 @@ class ControlBackend(object):
 
     @log_call(logger.debug, with_args=False)
     @inlineCallbacks
-    def login(self, email, password):
-        """Login using 'email' and 'password'."""
-        result = yield self.login_client.login_email_password(
-                    email=email, password=password)
+    def login(self, email=None, password=None):
+        """Login using 'email' and 'password' if not None, else prompt user."""
+        result = None
+        if email is not None and password is not None:
+            result = yield self.login_client.login_email_password(
+                        email=email, password=password)
+        else:
+            result = yield self.login_client.login()
+        # cache credentils
+        self._credentials = result
+        returnValue(result)
+
+    @log_call(logger.debug)
+    @inlineCallbacks
+    def register(self):
+        """Register a new user."""
+        result = yield self.login_client.register()
         # cache credentils
         self._credentials = result
         returnValue(result)
@@ -559,7 +592,7 @@ class ControlBackend(object):
 
     @log_call(logger.debug)
     @inlineCallbacks
-    def volumes_info(self, with_storage_info=True):
+    def volumes_info(self, with_storage_info=True, refresh=False):
         """Get the volumes info."""
         self._volumes = {}
 
@@ -572,15 +605,19 @@ class ControlBackend(object):
             else:
                 free_bytes = account['quota_total'] - account['quota_used']
 
+        if refresh:
+            yield self.sd_client.refresh_volumes()
+
         root_dir = yield self.sd_client.get_root_dir()
         shares_dir = yield self.sd_client.get_shares_dir()
         shares_dir_link = yield self.sd_client.get_shares_dir_link()
         folders = yield self.sd_client.get_folders()
         shares = yield self.sd_client.get_shares()
 
+        display_name = yield self._process_path(root_dir)
         root_volume = {u'volume_id': u'', u'path': root_dir,
                        u'subscribed': True, u'type': self.ROOT_TYPE,
-                       u'display_name': self._process_path(root_dir)}
+                       u'display_name': display_name}
         self._volumes[u''] = root_volume
 
         # group shares by the offering user
@@ -618,7 +655,8 @@ class ControlBackend(object):
                 logger.warning('volumes_info: udf %r already in the volumes '
                                'list (%r).', vid, self._volumes[vid])
             folder[u'subscribed'] = bool(folder[u'subscribed'])
-            folder[u'display_name'] = self._process_path(folder[u'path'])
+            display_name = yield self._process_path(folder[u'path'])
+            folder[u'display_name'] = display_name
             self._volumes[vid] = folder
 
         folders.sort(key=operator.itemgetter('path'))
@@ -675,28 +713,9 @@ class ControlBackend(object):
         yield self.sd_client.create_folder(path=folder_path)
 
     @log_call(logger.debug)
-    @inlineCallbacks
     def validate_path_for_folder(self, folder_path):
         """Validate 'folder_path' for folder creation."""
-        user_home = os.path.expanduser('~')
-        folder_path = append_path_sep(folder_path)
-
-        # handle folder_path not within '~' or links
-        # XXX is_link expects bytes, see bug #824252
-        if not folder_path.startswith(user_home) or is_link(
-            folder_path.encode('utf-8')):
-            returnValue(False)
-
-        # handle folder_path nested with a existing cloud folder
-        volumes = yield self.volumes_info(with_storage_info=False)
-        for _, _, data in volumes:
-            for volume in data:
-                cloud_folder = append_path_sep(volume['path'])
-                if (folder_path.startswith(cloud_folder) or
-                    cloud_folder.startswith(folder_path)):
-                    returnValue(False)
-
-        returnValue(True)
+        return self.sd_client.validate_path(folder_path)
 
     @log_call(logger.debug)
     @inlineCallbacks
@@ -745,8 +764,13 @@ class ControlBackend(object):
             result[name] = bool(value)
 
         limits = yield self.sd_client.get_throttling_limits()
-        result[DOWNLOAD_KEY] = limits['download']
-        result[UPLOAD_KEY] = limits['upload']
+        limits_enabled = yield self.sd_client.bandwidth_throttling_enabled()
+        if limits_enabled:
+            result[DOWNLOAD_KEY] = limits['download']
+            result[UPLOAD_KEY] = limits['upload']
+        else:
+            result[DOWNLOAD_KEY] = -1
+            result[UPLOAD_KEY] = -1
 
         returnValue(result)
 
