@@ -254,9 +254,10 @@ class DBusServiceTestCase(TestCase):
 
     timeout = 5
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize each test run."""
-        super(DBusServiceTestCase, self).setUp()
+        yield super(DBusServiceTestCase, self).setUp()
         dbus_service.init_mainloop()
         self._called = False
 
@@ -337,8 +338,9 @@ class BaseTestCase(TestCase):
 
     timeout = 3
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(BaseTestCase, self).setUp()
+        yield super(BaseTestCase, self).setUp()
         dbus_service.init_mainloop()
         self.patch(dbus_service, 'ControlBackend', MockBackend)
         be = dbus_service.publish_backend()
@@ -351,11 +353,6 @@ class BaseTestCase(TestCase):
                                       dbus_interface=DBUS_PREFERENCES_IFACE)
         self.deferred = defer.Deferred()
 
-    def tearDown(self):
-        self.backend = None
-        self.deferred = None
-        super(BaseTestCase, self).tearDown()
-
     def got_error(self, *a):
         """Some error happened in the DBus call."""
         self.deferred.errback(*a)
@@ -612,8 +609,9 @@ class OperationsTestCase(BaseTestCase):
 class OperationsErrorTestCase(OperationsTestCase):
     """Test for the DBus service operations when there is an error."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(OperationsErrorTestCase, self).setUp()
+        yield super(OperationsErrorTestCase, self).setUp()
         self.patch(MockBackend, 'exception', AssertionError)
 
     def assert_correct_method_call(self, success_sig, error_sig, success_cb,
@@ -644,8 +642,9 @@ class OperationsErrorTestCase(OperationsTestCase):
 class OperationsAuthErrorTestCase(OperationsTestCase):
     """Test for the DBus service operations when UnauthorizedError happens."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(OperationsAuthErrorTestCase, self).setUp()
+        yield super(OperationsAuthErrorTestCase, self).setUp()
         self.patch(MockBackend, 'exception',
                    dbus_service.UnauthorizedError)
 
