@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
-
-# Authors: Alejandro J. Cura <alecu@canonical.com>
-# Authors: Natalia B. Bidart <nataliabidart@canonical.com>
 #
-# Copyright 2010 Canonical Ltd.
+# Copyright 2010-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -43,6 +40,8 @@ from ubuntuone.controlpanel.backend import (
     FILE_SYNC_SYNCING,
     FILE_SYNC_UNKNOWN,
     MSG_KEY, STATUS_KEY,
+    UBUNTUONE_FROM_OAUTH,
+    UBUNTUONE_LINK,
 )
 from ubuntuone.controlpanel.tests import (TestCase,
     EMPTY_DESCRIPTION_JSON,
@@ -65,6 +64,12 @@ from ubuntuone.controlpanel.tests import (TestCase,
     USER_HOME,
 )
 
+SAMPLE_SIGNED = UBUNTUONE_FROM_OAUTH + '?oauth_nonce=' \
+    '36886134&oauth_timestamp=1310671062&oauth_consumer_key=consumer_key&' \
+    'oauth_signature_method=HMAC-SHA1&next=%2Fblah&oauth_version=1.0&' \
+    'oauth_token=GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo&' \
+    'oauth_signature=s6h0LRBiWchTADrTJWaJUSuaGpo%3D'
+
 
 # pylint: disable=E1101, W0201, W0212
 
@@ -104,6 +109,13 @@ class MockWebClient(CallRecorder):
             result = simplejson.loads(self.results[method])
             return defer.succeed(result)
 
+    @defer.inlineCallbacks
+    def build_signed_iri(self, iri, params):
+        """Fake the IRI signing."""
+        creds = yield self.get_credentials()
+        result = u'%s-%s-%s' % (iri, unicode(creds), unicode(params))
+        defer.returnValue(result)
+
 
 class MockLoginClient(CallRecorder):
     """A mock login_client module."""
@@ -154,6 +166,7 @@ class MockSDClient(CallRecorder):
         self.actions = []
         self.shares = []
         self.folders = []
+        self.volumes_refreshed = False
 
     def get_throttling_limits(self):
         """Return the sample speed limits."""
@@ -248,6 +261,10 @@ class MockSDClient(CallRecorder):
         """Stop the file_sync service."""
         self.actions.append('stop')
 
+    def get_home_dir(self):
+        """Grab the home dir."""
+        return USER_HOME
+
     def get_root_dir(self):
         """Grab the root dir."""
         return ROOT_PATH
@@ -272,6 +289,10 @@ class MockSDClient(CallRecorder):
         """Unsubcribe from 'volume_id'."""
         self.subscribed_folders.remove(volume_id)
 
+    def validate_path(self, path):
+        """Validate a path for folder creation."""
+        return path != USER_HOME
+
     def create_folder(self, path):
         """Grab list of folders."""
         self.folders.append(path)
@@ -300,6 +321,11 @@ class MockSDClient(CallRecorder):
         """Grab list of shared (shares from the user to others)."""
         return SAMPLE_SHARED
 
+    def refresh_volumes(self):
+        """Refresh the volume list."""
+        self.volumes_refreshed = True
+        return defer.succeed(None)
+
 
 class MockReplicationClient(CallRecorder):
     """A mock replication_client module."""
@@ -340,9 +366,10 @@ class BackendBasicTestCase(TestCase):
 
     timeout = 3
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(BackendBasicTestCase, self).setUp()
-        self.patch(backend, "web_client_factory", MockWebClient)
+        yield super(BackendBasicTestCase, self).setUp()
+        self.patch(backend, "WebClient", MockWebClient)
         self.patch(backend, "CredentialsManagementTool", MockLoginClient)
         self.patch(backend.sd_client, "SyncDaemonClient", MockSDClient)
         self.patch(backend, "replication_client", MockReplicationClient())
@@ -363,12 +390,14 @@ class BackendBasicTestCase(TestCase):
     @inlineCallbacks
     def test_get_token(self):
         """The get_token method returns the right token."""
+        self.patch(self.be, 'get_credentials', lambda: defer.succeed(TOKEN))
         token = yield self.be.get_token()
         self.assertEqual(token, TOKEN["token"])
 
     @inlineCallbacks
     def test_device_is_local(self):
         """The device_is_local returns the right result."""
+        self.patch(self.be, 'get_credentials', lambda: defer.succeed(TOKEN))
         result = yield self.be.device_is_local(self.local_token)
         self.assertTrue(result)
 
@@ -403,6 +432,54 @@ class BackendBasicTestCase(TestCase):
         self.assertIs(self.be.sd_client, self.be.sd_client)
 
 
+class SignIriTestCase(BackendBasicTestCase):
+    """Test cases for the IRI signing function."""
+
+    @defer.inlineCallbacks
+    def setUp(self):
+        yield super(SignIriTestCase, self).setUp()
+        self.patch(self.be, 'get_credentials', lambda: defer.succeed(TOKEN))
+
+    @inlineCallbacks
+    def test_without_ubuntuone_prefix(self):
+        """If given url is not an ubuntuone url, don't sign it."""
+        iri = 'bad_prefix' + UBUNTUONE_LINK
+        result = yield self.be.build_signed_iri(iri)
+
+        self.assertEqual(result, iri)
+
+    @inlineCallbacks
+    def test_with_ubuntuone_prefix(self):
+        """If given url is an ubuntuone url, sign it."""
+        iri = UBUNTUONE_LINK + 'foo'
+        result = yield self.be.build_signed_iri(iri)
+
+        expected = yield self.be.wc.build_signed_iri(UBUNTUONE_FROM_OAUTH,
+                                                     {'next': iri})
+        self.assertEqual(expected, result)
+
+
+class SignIriNoCredsTestCase(SignIriTestCase):
+    """The test suite for the sign url management when there are no creds."""
+
+    @defer.inlineCallbacks
+    def setUp(self):
+        yield super(SignIriNoCredsTestCase, self).setUp()
+        self.patch(self.be, 'get_credentials', lambda: defer.succeed({}))
+
+    @inlineCallbacks
+    def test_with_ubuntuone_prefix(self):
+        """If given url is an ubuntuone url, don't sign it.
+
+        Since we have no credentials, the url should not be signed.
+
+        """
+        iri = UBUNTUONE_LINK + 'foo'
+        result = yield self.be.build_signed_iri(iri)
+
+        self.assertEqual(result, iri)
+
+
 class BackendCredentialsTestCase(BackendBasicTestCase):
     """Credentials tests for the backend."""
 
@@ -775,23 +852,34 @@ class BackendVolumesTestCase(BackendBasicTestCase):
 
     # Access to a protected member of a client class
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(BackendVolumesTestCase, self).setUp()
+        yield super(BackendVolumesTestCase, self).setUp()
         # fake quota info and calculate free bytes
         self.be.wc.results[ACCOUNT_API] = SAMPLE_ACCOUNT_NO_CURRENT_PLAN
         self.be.wc.results[QUOTA_API] = SAMPLE_QUOTA_JSON
 
         # root dir info
+        display_name = yield self.be._process_path(ROOT_PATH)
         self.root_volume = {
             u'volume_id': u'', u'path': ROOT_PATH, u'subscribed': True,
             u'type': self.be.ROOT_TYPE,
-            u'display_name': self.be._process_path(ROOT_PATH),
+            u'display_name': display_name,
         }
 
+        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
+        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
+
     @inlineCallbacks
-    def expected_volumes(self, sample_shares, sample_folders,
+    def expected_volumes(self, sample_shares=None, sample_folders=None,
                          with_storage_info=True):
         """Get shares and group by sharing user, get folders and free space."""
+        if sample_shares is None:
+            sample_shares = self.be.sd_client.shares
+
+        if sample_folders is None:
+            sample_folders = self.be.sd_client.folders
+
         free_bytes = self.be.FREE_BYTES_NOT_AVAILABLE
         if with_storage_info:
             try:
@@ -829,7 +917,8 @@ class BackendVolumesTestCase(BackendBasicTestCase):
             folder = folder.copy()
             folder[u'type'] = self.be.FOLDER_TYPE
             folder[u'subscribed'] = bool(folder[u'subscribed'])
-            folder[u'display_name'] = self.be._process_path(folder[u'path'])
+            display_name = yield self.be._process_path(folder[u'path'])
+            folder[u'display_name'] = display_name
             folders.append(folder)
 
         # sort folders by path
@@ -853,21 +942,44 @@ class BackendVolumesTestCase(BackendBasicTestCase):
     @inlineCallbacks
     def test_volumes_info(self):
         """The volumes_info method exercises its callback."""
-        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
-
-        expected = yield self.expected_volumes(SAMPLE_SHARES, SAMPLE_FOLDERS)
+        expected = yield self.expected_volumes()
         result = yield self.be.volumes_info()
+
         self.assertEqual(result, expected)
+        self.assertFalse(self.be.sd_client.volumes_refreshed)
+
+    @inlineCallbacks
+    def test_volumes_info_can_refresh_volumes(self):
+        """The volumes_info can be refreshed."""
+        expected = yield self.expected_volumes()
+        result = yield self.be.volumes_info(refresh=True)
+
+        self.assertTrue(self.be.sd_client.volumes_refreshed)
+        self.assertEqual(result, expected)
+
+    @inlineCallbacks
+    def test_volumes_info_process_path(self):
+        """The volumes_info method exercises its callback."""
+        root_path = USER_HOME + os.path.sep + 'My Ubuntu' + USER_HOME
+        self.patch(self.be.sd_client, 'get_root_dir', lambda: root_path)
+        for item in SAMPLE_FOLDERS:
+            path = item[u'path']
+            path = path[len(USER_HOME) + 1:]
+            item[u'path'] = os.path.join(root_path, path)
+        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
+
+        yield self.be.volumes_info()
+        for item in SAMPLE_FOLDERS:
+            key = item[u'volume_id']
+            folder = self.be._volumes[key]
+            display_name = folder['display_name']
+            prefix = 'My Ubuntu' + USER_HOME
+            self.assertTrue(display_name.startswith(prefix))
 
     @inlineCallbacks
     def test_volumes_info_without_storage_info(self):
         """The volumes_info method exercises its callback."""
-        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
-
-        expected = yield self.expected_volumes(SAMPLE_SHARES, SAMPLE_FOLDERS,
-                                               with_storage_info=False)
+        expected = yield self.expected_volumes(with_storage_info=False)
         result = yield self.be.volumes_info(with_storage_info=False)
         self.assertEqual(result, expected)
 
@@ -878,11 +990,8 @@ class BackendVolumesTestCase(BackendBasicTestCase):
     @inlineCallbacks
     def test_volumes_are_cached(self):
         """The volume list is cached."""
-        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
-
         expected = {}
-        info = yield self.expected_volumes(SAMPLE_SHARES, SAMPLE_FOLDERS)
+        info = yield self.expected_volumes()
         for _, _, data in info:
             for volume in data:
                 sid = volume['volume_id']
@@ -931,10 +1040,8 @@ class BackendVolumesTestCase(BackendBasicTestCase):
         ]
 
         self.patch(self.be.sd_client, 'shares', read_only_shares)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
 
-        expected = yield self.expected_volumes(read_only_shares,
-                                               SAMPLE_FOLDERS)
+        expected = yield self.expected_volumes()
         result = yield self.be.volumes_info()
         self.assertEqual(result, expected)
 
@@ -942,10 +1049,8 @@ class BackendVolumesTestCase(BackendBasicTestCase):
     def test_volumes_info_no_quota_for_root(self):
         """The volumes_info returns info even if quota call fails."""
         self.be.wc.failure = 500
-        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
 
-        expected = yield self.expected_volumes(SAMPLE_SHARES, SAMPLE_FOLDERS)
+        expected = yield self.expected_volumes()
         result = yield self.be.volumes_info()
 
         self.assertEqual(len(result), len(expected))
@@ -1033,28 +1138,15 @@ class BackendVolumesTestCase(BackendBasicTestCase):
     @inlineCallbacks
     def test_create_folder(self):
         """New folders can be created."""
+        self.patch(self.be.sd_client, 'folders', [])
+
         folder_path = os.path.join(USER_HOME, 'Test Me')
         yield self.be.create_folder(folder_path=folder_path)
 
         self.assertEqual(self.be.sd_client.folders, [folder_path])
 
-
-class BackendValidatePathForFolderTestCase(BackendBasicTestCase):
-    """Test for the path validator when creating folders."""
-
-    msg = '%r must not be a valid path for creating a folder.'
-
-    def setUp(self):
-        super(BackendValidatePathForFolderTestCase, self).setUp()
-        old_home = os.environ['HOME']
-        os.environ['HOME'] = USER_HOME
-        self.addCleanup(os.environ.__setitem__, 'HOME', old_home)
-        self.patch(self.be.sd_client, 'shares', SAMPLE_SHARES)
-        self.patch(self.be.sd_client, 'folders', SAMPLE_FOLDERS)
-        self.patch(backend, 'is_link', lambda p: False)
-
     @defer.inlineCallbacks
-    def test_valid_if_folder_inside_home(self):
+    def test_validate_path_for_folder(self):
         """Test proper validation of path for creating folders."""
         folder_path = os.path.join(USER_HOME, 'Test Me')
 
@@ -1063,68 +1155,11 @@ class BackendValidatePathForFolderTestCase(BackendBasicTestCase):
             '%r must be a valid path for creating a folder.' % folder_path)
 
     @defer.inlineCallbacks
-    def test_valid_if_folder_shares_a_prefix_with_an_udf(self):
-        """Test proper validation of path for creating folders.
-
-        If the user chooses a folder with the same prefix as an UDF, but
-        outside every UDF, the path is valid.
-
-        """
-        tricky_path = ROOT_PATH
-        assert not tricky_path.endswith(os.path.sep)
-        tricky_path += ' Suffix'
-        assert tricky_path.startswith(ROOT_PATH)
-
-        result = yield self.be.validate_path_for_folder(tricky_path)
-        self.assertTrue(result,
-            '%r must be a valid path for creating a folder.' % tricky_path)
-
-    @inlineCallbacks
-    def test_path_not_valid_if_outside_home(self):
-        """A folder outside ~ is not valid."""
-        outside_home = os.path.abspath(os.path.join(USER_HOME, os.path.pardir))
-
-        result = yield self.be.validate_path_for_folder(outside_home)
-        self.assertFalse(result, self.msg % outside_home)
-
-    @defer.inlineCallbacks
-    def test_not_valid_if_folder_inside_root(self):
-        """A folder inside the root is not valid."""
-        root_path = ROOT_PATH
-        # create a valid path inside the root
-        inside_root = os.path.abspath(os.path.join(root_path, 'test'))
-
-        result = yield self.be.validate_path_for_folder(inside_root)
-        self.assertFalse(result, self.msg % inside_root)
-
-    @defer.inlineCallbacks
-    def test_not_valid_if_folder_inside_an_udf(self):
-        """A folder inside an UDF is not valid."""
-        udf_path = SAMPLE_FOLDERS[-1]['path']
-        # create a valid path inside an existing UDF
-        inside_udf = os.path.abspath(os.path.join(udf_path, 'test'))
-
-        result = yield self.be.validate_path_for_folder(inside_udf)
-        self.assertFalse(result, self.msg % inside_udf)
-
-    @defer.inlineCallbacks
-    def test_not_valid_if_folder_is_parent_of_an_udf(self):
-        """A folder parent of an UDF is not valid."""
-        udf_path = SAMPLE_FOLDERS[-1]['path']
-        # create a valid path that is parent from an existing UDF
-        udf_parent = os.path.abspath(os.path.join(udf_path, os.path.pardir))
-
-        result = yield self.be.validate_path_for_folder(udf_parent)
-        self.assertFalse(result, self.msg % udf_parent)
-
-    @defer.inlineCallbacks
-    def test_not_valid_if_folder_is_link(self):
-        """A link path is not valid."""
-        self.patch(backend, 'is_link', lambda p: True)
-        path_link = os.path.join(USER_HOME, 'Test Me')
-
-        result = yield self.be.validate_path_for_folder(path_link)
-        self.assertFalse(result, self.msg % path_link)
+    def test_validate_path_for_folder_invalid(self):
+        """Test proper validation of path for creating folders."""
+        result = yield self.be.validate_path_for_folder(USER_HOME)
+        self.assertFalse(result,
+            '%r must not be a valid path for creating a folder.' % USER_HOME)
 
 
 class BackendSyncStatusTestCase(BackendBasicTestCase):
@@ -1132,8 +1167,9 @@ class BackendSyncStatusTestCase(BackendBasicTestCase):
 
     was_disabled = False
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(BackendSyncStatusTestCase, self).setUp()
+        yield super(BackendSyncStatusTestCase, self).setUp()
         self.be.file_sync_disabled = self.was_disabled
 
     def _build_msg(self):
@@ -1353,8 +1389,9 @@ class BackendSyncStatusIfDisabledTestCase(BackendSyncStatusTestCase):
 class BackendFileSyncOpsTestCase(BackendBasicTestCase):
     """Syncdaemon operations for the backend."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        super(BackendFileSyncOpsTestCase, self).setUp()
+        yield super(BackendFileSyncOpsTestCase, self).setUp()
         self.be.sd_client.actions = []
 
     @inlineCallbacks
@@ -1502,6 +1539,7 @@ class BackendFileSyncSettingsTestCase(BackendBasicTestCase):
     @inlineCallbacks
     def test_file_sync_settings_info(self):
         """The settings_info method exercises its callback."""
+        self.patch(self.be.sd_client, "throttling", True)
         self.be.sd_client.limits = {"download": 1000, "upload": 100}
         expected = {
             backend.AUTOCONNECT_KEY: self.be.sd_client.autoconnect,
@@ -1517,6 +1555,25 @@ class BackendFileSyncSettingsTestCase(BackendBasicTestCase):
         result = yield self.be.file_sync_settings_info()
         self.assertEqual(expected, result)
 
+    @inlineCallbacks
+    def test_file_sync_settings_info_with_limit(self):
+        """The settings_info method exercises its callback."""
+        self.patch(self.be.sd_client, "throttling", False)
+        self.be.sd_client.limits = {"download": 987456, "upload": 125698}
+        expected = {
+            backend.AUTOCONNECT_KEY: self.be.sd_client.autoconnect,
+            backend.SHOW_ALL_NOTIFICATIONS_KEY:
+                self.be.sd_client.show_all_notifications,
+            backend.SHARE_AUTOSUBSCRIBE_KEY:
+                self.be.sd_client.share_autosubscribe,
+            backend.UDF_AUTOSUBSCRIBE_KEY:
+                self.be.sd_client.udf_autosubscribe,
+            backend.DOWNLOAD_KEY: -1,
+            backend.UPLOAD_KEY: -1,
+        }
+        result = yield self.be.file_sync_settings_info()
+        self.assertEqual(expected, result)
+
     @inlineCallbacks
     def test_change_file_sync_setting_autoconnect(self):
         """The settings can be changed for autoconnect."""
